; BlinkenBadge
;==============
;
; ATMega128L-8 @ 8MHz internal R/C or crystal
;
;PORT PINS:
;  PB0-7 = LED Col 1..8
;  PC4-7 = LED Row 5..8
;  PD0-7 = LED Col 11..18
;  PE0   = RS232 Rx
;  PE1   = RS232 Tx
;  PE2-5 = LED Row 1..4
;  PG3   = LED Col 9
;  PG4   = LED Col 10
;
;VERSION HISTORY:
;  2005-12-13   v0.01   (Arne Rossius)
;  2006-03-11   v0.1   (Arne Rossius)

;===============================================================================

.def	temp = R16
.def	temp2 = R17
.def	row = R18
.def	col1 = R19
.def	col2 = R20
.def	col3 = R21

.def	durL = R24
.def	durH = R25

;===============================================================================

.include "../include/m128def.inc"
.cseg
.org 0x0000
	rjmp	start

.org OVF0addr ;timer 0 overflow (~1000 Hz)
	rjmp	ovf0int

;===============================================================================

ovf0int: ;timer 0 overflow int (~1000 Hz)
	;backup registers
	push	temp
	push	temp2
	in	temp, SREG
	push	temp

	;decrement frame duration counter
	sbiw	durH:durL, 1
	brne	ovf0int_end

	;display next frame
	push	YL
	push	YH
nextframe:
	lpm	durL, Z+
	lpm	durH, Z+
	cpi	durL, 0
	cpc	durH, durL
	breq	restart
	ldi	YL, LOW(0x100)
	ldi	YH, HIGH(0x100)
	ldi	temp2, 3*8
flash2ram:
	lpm	temp, Z+
	st	Y+, temp
	dec	temp2
	brne	flash2ram
	pop	YH
	pop	YL

	;return
ovf0int_end:
	pop	temp
	out	SREG, temp
	pop	temp2
	pop	temp
	reti

restart:
	;restart movie sequence
	ldi	ZL, LOW(chaosknoten*2)
	ldi	ZH, HIGH(chaosknoten*2)
	rjmp	nextframe

;===============================================================================

start:
	;setup stack pointer
	ldi	temp, LOW(RAMEND)
	out	SPL, temp
	ldi	temp, HIGH(RAMEND)
	out	SPH, temp

	;setup ports
	ldi	temp, 0x00 ;PORTA = inputs with pullups (unused)
	out	DDRA, temp
	ldi	temp, 0xFF
	out	PORTA, temp
	ldi	temp, 0xFF ;PORTB = outputs (Cols)
	out	DDRB, temp
	ldi	temp, 0x00
	out	PORTB, temp
	ldi	temp, 0xFF ;PORTC = outputs, 0-3 unused, 4-7 (Rows)
	out	DDRC, temp
	ldi	temp, 0x00
	out	PORTC, temp
	ldi	temp, 0xFF ;PORTD = outputs (Cols)
	out	DDRD, temp
	ldi	temp, 0x00
	out	PORTD, temp
	ldi	temp, 0x3C ;PORTE = 0,1+6,7 input with pullups (ISP/unused), 2-5 output (Rows)
	out	DDRE, temp
	ldi	temp, 0xC3
	out	PORTE, temp
	ldi	temp, 0x00 ;PORTF = inputs with pullups (unused)
	sts	DDRF, temp
	ldi	temp, 0xFF
	sts	PORTF, temp
	ldi	temp, 0x1F ;PORTG = outputs, 3,4 (Cols), 5-7 N/A
	sts	DDRG, temp
	ldi	temp, 0x00
	sts	PORTG, temp

	;setup timer
	ldi	temp, 1<<CS00 | 1<<CS01 ;Clk/32
	ldi	temp, 0x03
	out	TCCR0, temp
	ldi	temp, 1<<TOIE0 ;enable overflow INT
	out	TIMSK, temp

	;preload first frame address
	ldi	ZL, LOW(chaosknoten*2)
	ldi	ZH, HIGH(chaosknoten*2)
	ldi	durL, 1
	ldi	durH, 0

	;enable interrupts
	sei

;-------------------

loop:
	;repeatedly display frame from RAM
	ldi	YL, LOW(0x100)
	ldi	YH, HIGH(0x100)
	rcall	get_line ;line 1
	sbi	PORTE, 2
	rcall	mpxwait
	cbi	PORTE, 2
	rcall	get_line ;line 2
	sbi	PORTE, 3
	rcall	mpxwait
	cbi	PORTE, 3
	rcall	get_line ;line 3
	sbi	PORTE, 4
	rcall	mpxwait
	cbi	PORTE, 4
	rcall	get_line ;line 4
	sbi	PORTE, 5
	rcall	mpxwait
	cbi	PORTE, 5
	rcall	get_line ;line 5
	sbi	PORTC, 4
	rcall	mpxwait
	cbi	PORTC, 4
	rcall	get_line ;line 6
	sbi	PORTC, 5
	rcall	mpxwait
	cbi	PORTC, 5
	rcall	get_line ;line 7
	sbi	PORTC, 6
	rcall	mpxwait
	cbi	PORTC, 6
	rcall	get_line ;line 8
	sbi	PORTC, 7
	rcall	mpxwait
	cbi	PORTC, 7
	rjmp	loop

;-------------------

mpxwait:
	;wait some time (used for multiplexing)
	ldi	temp, 0x00
mpxwait_loop:
	dec	temp
	brne	mpxwait_loop
	ret

;-------------------

get_line:
	;display one line from RAM
	ld	col1, Y+
	ld	col2, Y+
	andi	col2, 0x1F
	ori	col2, 0x07
	ld	col3, Y+
	out	PORTB, col1
	sts	PORTG, col2
	out	PORTD, col3
	ret

;===============================================================================

	;movie frames
	;format:
	;   * frames
	;      * 2 byte duration (LSB first), milliseconds
	;      * 8x 3 byte per display line (usage: 0xFF 0x18 0xFF), LSB = left pixel
	;   * end marker (0x00 0x00)
chaosknoten:
	.dw 1000
	.db 0xFE, 0x08, 0x00, 0x51, 0x10, 0x00
	.db 0xE1, 0x18, 0x3F, 0xE1, 0x18, 0x77
	.db 0x51, 0x10, 0x38, 0xFE, 0x08, 0x30
	.db 0x00, 0x00, 0x58, 0x00, 0x00, 0x94
	.include "movies.asm"
	.dw 0 ;end marker